"use strict";
const proxyquire = require("proxyquire");
const assert = require("power-assert");
const sinon = require("sinon");
const connectStub = sinon.spy();
const MongoStub = {
    MongoClient: {
        connect: () => { }
    }
};
const MongoConnector = proxyquire('../mongo_connector', { 'mongodb': MongoStub }).default;
describe('connector', () => {
    it('will initialise url', () => {
        const url = 'mongodb://url';
        const spy = sinon.spy();
        MongoStub.MongoClient.connect = () => spy();
        const connector = new MongoConnector(url);
        connector.connect();
        assert.equal(connector.mongoUrl, url);
        assert(spy.calledOnce);
    });
    it('will not initialise db on error and logs error', sinon.test(function () {
        const url = 'mongodb://url';
        const successSpy = sinon.spy();
        const dirStub = this.stub(console, 'dir');
        MongoStub.MongoClient.connect = (url, func) => func('error');
        const connector = new MongoConnector(url);
        connector.connect();
        assert.equal(connector.db, undefined);
        sinon.assert.calledOnce(dirStub);
        sinon.assert.calledWith(dirStub, 'error');
    }));
    it('will initialise and calls back', () => {
        const url = 'mongodb://url';
        const db = {};
        const startedSpy = sinon.spy();
        MongoStub.MongoClient.connect = (url, func) => func(null, db);
        const connector = new MongoConnector(url, startedSpy);
        assert.equal(connector.db, db);
        assert(startedSpy.calledOnce);
    });
    it('will create a new collection', () => {
        const url = '';
        const db = {
            collection: sinon.spy()
        };
        MongoStub.MongoClient.connect = (url, func) => func(null, db);
        const connector = new MongoConnector(url);
        connector.connect();
        assert.equal(connector.db, db);
        const collectionName = 'Col';
        const col = connector.collection(collectionName);
        assert(db.collection.calledWith(collectionName));
    });
});
